searchData={"content_type":"text/markdown","items":[{"doc":"Params provide some helpers method to work with parameters","ref":"Tarams.html","title":"Tarams","type":"module"},{"doc":"Cast and validate params with given schema.\nSee `Tarams.Schema` for instruction on how to define a schema\nAnd then use it like this\n\n```elixir\ndef index(conn, params) do\n  index_schema = %{\n    status: [type: :string, required: true],\n    type: [type: :string, in: [\"type1\", \"type2\", \"type3\"]],\n    keyword: [type: :string, length: [min: 3, max: 100]],\n  }\n\n  with {:ok, data}   IO.puts(errors)\n  end\nend\n```","ref":"Tarams.html#cast/2","title":"Tarams.cast/2","type":"function"},{"doc":"","ref":"Tarams.html#cast!/2","title":"Tarams.cast!/2","type":"function"},{"doc":"","ref":"Tarams.html#cast_array/3","title":"Tarams.cast_array/3","type":"function"},{"doc":"","ref":"Tarams.html#clean_nil/1","title":"Tarams.clean_nil/1","type":"function"},{"doc":"","ref":"Tarams.html#plug_scrub/2","title":"Tarams.plug_scrub/2","type":"function"},{"doc":"","ref":"Tarams.html#scrub_param/1","title":"Tarams.scrub_param/1","type":"function"},{"doc":"This is only a specification to define a schema to use with `Tarams.Params.cast` and `Tarams.Contract.validate`\n\nSchema is just a map or keyword list that follow some simple conventions. Map's key is the field name and the value is a keyword list of field specifications.\n\n**Example**\n\n```elixir\n%{\n  name: [type: :string, format: ~r/{4}/],\n  age: [type: :integer, number: [min: 15, max: 50]].\n  skill: [type: {:array, :string}, length: [min: 1, max: 10]]\n}\n```\n\nIf you only specify type of data, you can write it shorthand style like this\n\n```elixir\n%{\n  name: :string,\n  age: :integer,\n  skill: {:array, :string}\n}\n```","ref":"Tarams.Schema.html","title":"Tarams.Schema","type":"module"},{"doc":"**Built-in types**\n\nA type could be any of built-in supported types:\n\n- `boolean`\n- `string` | `binary`\n- `integer`\n- `float`\n- `number` (integer or float)\n- `date`\n- `time`\n- `datetime` | `utc_datetime`: date time with time zone\n- `naive_datetime`: date time without time zone\n- `map`\n- `keyword`\n- `{array, type}` array of built-in type, all item must be the same type\n\n\n**Other types**\nCustom type may be supported depends on module.\n\n**Nested types**\nNested types could be a another **schema** or list of **schema**\n\n\n```elixir\n%{\n  user: [type: %{\n      name: [type: :string]\n    }]\n}\n```\n\nOr list of schema\n\n```elixir\n%{\n  users: [type: {:array, %{\n      name: [type: :string]\n    }} ]\n}\n```\n\n**Alias**\nAlias allow set a new key for value when using with `Taram.cast`\n\n```elixir\nschema = %{\n  email: [type: :string, as: :user_email]\n}\n\nTarams.cast(%{email: \"xx@yy.com\"}, schema)\n#> {:ok, %{user_email: \"xx@yy.com\"}}\n```","ref":"Tarams.Schema.html#module-i-field-type","title":"I. Field type - Tarams.Schema","type":"module"},{"doc":"These specifications is used for casting data with `Tarams.Params.cast`\n\n#","ref":"Tarams.Schema.html#module-ii-field-casting-and-default-value","title":"II. Field casting and default value - Tarams.Schema","type":"module"},{"doc":"Is used when the given field is missing or nil.\n\n- Default could be a value\n\n  ```elixir\n  %{\n    status: [type: :string, default: \"active\"]\n  }\n  ```\n\n- Or a `function/0`, this function will be invoke each time data is `casted`\n\n  ```elixir\n  %{\n    published_at: [type: :datetime, default: &DateTime.utc_now/0]\n  }\n  ```\n\n#","ref":"Tarams.Schema.html#module-1-default-value","title":"1. Default value - Tarams.Schema","type":"module"},{"doc":"You can provide a function to cast field value instead of using default casting function by using\n`cast_func:  `\n\n```elixir\n%{\n    published_at: [type: :datetime, cast_func: &DateTime.from_iso8601/1]\n}\n```","ref":"Tarams.Schema.html#module-2-custom-cast-function","title":"2. Custom cast function - Tarams.Schema","type":"module"},{"doc":"**These validation are supported by `Tarams.Validator`**\n\n#","ref":"Tarams.Schema.html#module-iii-field-validation","title":"III. Field validation - Tarams.Schema","type":"module"},{"doc":"Type specification above could be used for validating or casting data.\n\n#","ref":"Tarams.Schema.html#module-1-type-validation","title":"1. Type validation - Tarams.Schema","type":"module"},{"doc":"Support validating number value. These are list of supported validations:\n\n  - `equal_to`\n  - `greater_than_or_equal_to` | `min`\n  - `greater_than`\n  - `less_than`\n  - `less_than_or_equal_to` | `max`\n\n Define validation: `number: [ :  , ...]`\n\n**Example**\n\n```elixir\n%{\n  age: [type: :integer, number: [min: 1, max: 100]]\n}\n```\n\n#","ref":"Tarams.Schema.html#module-2-numeric-validation","title":"2. Numeric validation - Tarams.Schema","type":"module"},{"doc":"Validate length of supported types include `string`, `array`, `map`, `tuple`, `keyword`.\nLength condions are the same with **Numeric validation**\n\nDefine validation: `length: [ :  , ...]`\n\n**Example**\n\n```elixir\n%{\n  skills: [type: {:array, :string}, length: [min: 0, max: 5]]\n}\n```\n\n\n#","ref":"Tarams.Schema.html#module-3-length-validation","title":"3. Length validation - Tarams.Schema","type":"module"},{"doc":"Check if a string match a given pattern.\n\nDefine validation: `format:  `\n\n**Example**\n\n```elixir\n%{\n  year: [type: :string, format: ~r/year: {4}/]\n}\n```\n\n#","ref":"Tarams.Schema.html#module-4-format-validation","title":"4. Format validation - Tarams.Schema","type":"module"},{"doc":"Check if value is included or not included in given enumerable (`array`, `map`, or `keyword`)\n\nDefine validation: `in:  ` or `not_in:  `\n\n**Example**\n\n```elixir\n%{\n  status: [type: :string, in: [\"active\", \"inactive\"]],\n  selected_option: [type: :integer, not_in: [2,3]]\n}\n```\n\n#","ref":"Tarams.Schema.html#module-5-inclusion-and-exclusion-validation","title":"5. Inclusion and exclusion validation - Tarams.Schema","type":"module"},{"doc":"You can provide a function to validate the value.\n\nDefine validation: `func:  `\n\nFunction must be follow this signature\n\n```elixir\n@spec func(value::any()) :: :ok | {:error, message::String.t()}\n```","ref":"Tarams.Schema.html#module-6-custom-validation-function","title":"6. Custom validation function - Tarams.Schema","type":"module"},{"doc":"Expand short-hand type syntax to full syntax\n\n    field: :string -> field: [type: :string]\n    field: {:array, :string} -> field: [type: {:array, :string}]\n    field: %{#embedded} -> field: [type: %{#embedded}]","ref":"Tarams.Schema.html#expand/1","title":"Tarams.Schema.expand/1","type":"function"},{"doc":"Cast data to target type.\nMuch code of this module is borrowed from `Ecto.Type`","ref":"Tarams.Type.html","title":"Tarams.Type","type":"module"},{"doc":"","ref":"Tarams.Type.html#cast/2","title":"Tarams.Type.cast/2","type":"function"},{"doc":"","ref":"Tarams.Type.html#cast!/2","title":"Tarams.Type.cast!/2","type":"function"},{"doc":"","ref":"Tarams.Utils.html","title":"Tarams.Utils","type":"module"},{"doc":"Clean all nil field from params, support nested map and list.\n\n**Example**\n\n    params = %{\"keyword\" => nil, \"email\" => nil, \"type\" => \"customer\"}\n    Tarams.clean_nil(params)\n    # => %{\"type\" => \"customer\"}\n\n    params = %{user_ids: [1, 2, nil]}\n    Tarams.clean_nil(params)\n    # => %{user_ids: [1, 2]}","ref":"Tarams.Utils.html#clean_nil/1","title":"Tarams.Utils.clean_nil/1","type":"function"},{"doc":"A plug which do srubbing params\n\n**Use in Router**\n\n    defmodule MyApp.Router do\n      ...\n      plug Tarams.plug_scrub\n      ...\n    end\n\n**Use in controller**\n\n    plug Tarams.plug_scrub when action in [:index, :show]\n    # or specify which field to scrub\n    plug Tarams.plug_scrub, [\"id\", \"keyword\"] when action in [:index, :show]","ref":"Tarams.Utils.html#plug_scrub/2","title":"Tarams.Utils.plug_scrub/2","type":"function"},{"doc":"Convert all parameter which value is empty string or string with all whitespace to nil. It works with nested map and list too.\n\n**Example**\n\n    params = %{\"keyword\" => \"   \", \"email\" => \"\", \"type\" => \"customer\"}\n    Tarams.scrub_param(params)\n    # => %{\"keyword\" => nil, \"email\" => nil, \"type\" => \"customer\"}\n\n    params = %{user_ids: [1, 2, \"\", \"  \"]}\n    Tarams.scrub_param(params)\n    # => %{user_ids: [1, 2, nil, nil]}","ref":"Tarams.Utils.html#scrub_param/1","title":"Tarams.Utils.scrub_param/1","type":"function"},{"doc":"# Tarams\n\nPhoenix request params validation library.\n\n[![Build Status](https://github.com/bluzky/tarams/workflows/Elixir%20CI/badge.svg)](https://github.com/bluzky/tarams/actions) [![Coverage Status](https://coveralls.io/repos/github/bluzky/tarams/badge.svg?branch=master)](https://coveralls.io/github/bluzky/tarams?branch=master) [![Hex Version](https://img.shields.io/hexpm/v/tarams.svg)](https://hex.pm/packages/tarams) [![docs](https://img.shields.io/badge/docs-hexpm-blue.svg)](https://hexdocs.pm/tarams/)\n\n\n**Warning: Tarams v1.0.0 APIs is not back compatible**\n\n- [Tarams](#tarams)\n    - [Why Tarams](#why-tarams)\n    - [Installation](#installation)\n    - [Usage](#usage)\n    - [Define schema](#define-schema)\n        - [Default value](#default-value)\n        - [Custom cast function](#custom-cast-function)\n            - [1. Custom cast function accept value only](#1-custom-cast-fuction-accept-value-only)\n            - [2. Custom cast function accept value and current object](#2-custom-cast-function-accept-value-and-current-object)\n            - [3.Custom cast function accept tuple {M, f}](#3custom-cast-function-accept-tuple-m-f)\n        - [Nested schema](#nested-schema)\n    - [Transform data](#transform-data)\n        - [Field name alias](#field-name-alias)\n        - [Convert data](#convert-data)\n    - [Validation](#validation)\n    - [Contributors](#contributors)","ref":"readme.html","title":"Tarams","type":"extras"},{"doc":"- Reduce code boilerplate \n    - Shorter schema definition\n    - Default function which generate value each casting time\n    - Custom validation functions\n    - Custom parse functions","ref":"readme.html#why-tarams","title":"Why Tarams - Tarams","type":"extras"},{"doc":"[Available in Hex](https://hex.pm/tarams), the package can be installed\nby adding `tarams` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:tarams, \"~> 1.0.0\"}\n  ]\nend\n```","ref":"readme.html#installation","title":"Installation - Tarams","type":"extras"},{"doc":"**Process order**\n> Cast data -> validate casted data -> transform data\n\n```elixir\n@index_params_schema  %{\n    keyword: :string,\n    status: [type: :string, required: true],\n    group_id: [type: :integer, number: [greater_than: 0]],\n    name: [type: :string, from: :another_field]\n  }\n\ndef index(conn, params) do\n    with {:ok, better_params}   # return params error\n    end\nend\n```","ref":"readme.html#usage","title":"Usage - Tarams","type":"extras"},{"doc":"Schema is just a map and it can be nested. Each field is defined as\n\n` : [ , ...]`\n\nOr short form\n\n` :  `\n\nField specs is a keyword list thay may include:\n\n- `type` is required, `Tarams` support same data type as `Ecto`. I borrowed code from Ecto\n- `default`: default value or default function\n- `cast_func`: custom cast function\n- `number, format, length, in, not_in, func, required, each` are available validations\n- `from`: use value from another field\n- `as`: alias key you will receive from `Tarams.cast` if casting is succeeded\n\n\n#","ref":"readme.html#define-schema","title":"Define schema - Tarams","type":"extras"},{"doc":"You can define a default value for a field if it's missing from the params.\n\n```elixir\nschema = %{\n    status: [type: :string, default: \"pending\"]\n}\n```\n\nOr you can define a default value as a function. This function is evaluated when `Tarams.cast` gets invoked.\n\n```elixir\nschema = %{\n    date: [type: :utc_datetime, default: &Timex.now/0]\n}\n```\n\n#","ref":"readme.html#default-value","title":"Default value - Tarams","type":"extras"},{"doc":"You can define your own casting function, `tarams` provide `cast_func` option.\nYour `cast_func` must follows this spec \n\n##","ref":"readme.html#custom-cast-function","title":"Custom cast function - Tarams","type":"extras"},{"doc":"```elixir\nfn(any) :: {:ok, any} | {:error, binary} | :error\n```\n\n```elixir\ndef my_array_parser(value) do\n    if is_binary(value) do\n        ids = \n            String.split(value, \",\")\n            |> Enum.map(&String.to_integer(&1))\n        \n        {:ok, ids}\n    else\n        {:error, \"Invalid string\"\n    end\nend\n\nschema = %{\n    user_id: [type: {:array, :integer}, cast_func: &my_array_parser/1]\n}\n\nTarams.cast(%{user_id: \"1,2,3\"}, schema)\n```\nThis is a demo parser function.\n\n##","ref":"readme.html#1-custom-cast-fuction-accept-value-only","title":"1. Custom cast fuction accept value only - Tarams","type":"extras"},{"doc":"```elixir\ndata = %{\n   name: \"tada\",\n   bold: true\n}\n\nschema = %{\n    name: [type: :string, cast_func: fn value, data -> \n        {:ok, (if data.bold, do: String.upcase(value), else: value)}\n    end]\n}\n\nTarams.cast(data, schema)\n\n# > %{name: \"TADA\"}\n```\n\n##","ref":"readme.html#2-custom-cast-function-accept-value-and-current-object","title":"2. Custom cast function accept value and current object - Tarams","type":"extras"},{"doc":"Your cast function must accept 2 arguments\n\n```elixir\ndefmodule MyModule do\n    def upcase(value, data) do\n        {:ok, (if data.bold, do: String.upcase(value), else: value)}\n    end\nend\n```\n\n```elixir\ndata = %{\n   name: \"tada\",\n   bold: true\n}\n\nschema = %{\n    name: [type: :string, cast_func: {MyModule, :upcase}]\n}\n\nTarams.cast(data, schema)\n\n# > %{name: \"TADA\"}\n```\n\n\n#","ref":"readme.html#3-custom-cast-function-accept-tuple-m-f","title":"3.Custom cast function accept tuple {M, f} - Tarams","type":"extras"},{"doc":"With `Tarams` you can parse and validate nested map and list easily\n\n```elixir\n@my_schema %{\n    status: :string,\n    pagination: %{\n        page: [type: :integer, number: [min: 1]],\n        size: [type: :integer, number: [min: 10, max: 100\"]]\n    }\n}\n```\n\nOr nested list schema\n\n```elixir\n@user_schema %{\n    name: :string,\n    email: [type: :string, required: true]\n    addresses: [type: {:array, %{\n        street: :string,\n        district: :string,\n        city: :string\n    }}]\n}\n```","ref":"readme.html#nested-schema","title":"Nested schema - Tarams","type":"extras"},{"doc":"`Tarams` uses `Valdi` validation library. You can read more about [Valdi here](https://github.com/bluzky/valdi)\nBasically it supports following validation\n\n- validate inclusion/exclusion\n- validate length for string and enumerable types\n- validate number\n- validate string format/pattern\n- validate custom function\n- validate required(not nil) or not\n- validate each array item\n\n\n\n  ```elixir\n  product_schema = %{\n    sku: [type: :string, required: true, length: [min: 6, max: 20]]\n    name: [type: :string, required: true],\n    quantity: [type: :integer, number: [min: 0]],\n    type: [type: :string, in: ~w(physical digital)],\n    expiration_date: [type: :naive_datetime, func: &my_validation_func/1],\n    # dynamic required\n    width: [type: :integer, required: fn value, data -> data.type == \"physical\" end],\n    # validate each array item\n    tags: [type: {:array, :string}, each: [length: [max: 50]]]\n  }\n  ```\n\n#","ref":"readme.html#validation","title":"Validation - Tarams","type":"extras"},{"doc":"- Can accept function or `{module, function}` tuple\n- Only support 2 arity function\n\n\n```elixir\ndef require_email?(value, data), do: is_nil(email.phone)\n\n....\n\n%{\n    phone: :string\n    name: [type: :string, required: fn value, data -> true end],\n    email: [type: :string, required: {__MODULE__, :require_email?}]\n}\n```\n\n#","ref":"readme.html#dynamic-required","title":"Dynamic required - Tarams","type":"extras"},{"doc":"Support validate array item with `:each` option, `each` accept a list of validators\n\n```elixir\n%{\n    values: [type: {:array, :number}, each: [number: [min: 20, max: 50]]]\n  }\n```","ref":"readme.html#validate-array-item","title":"Validate array item - Tarams","type":"extras"},{"doc":"#","ref":"readme.html#transform-data","title":"Transform data - Tarams","type":"extras"},{"doc":"You can set alias name for schema fields\n```elixir\ndata = %{\n   name: \"tada\"\n}\n\nschema = %{\n    name: [type: :string, as: :full_name]\n}\n\nTarams.cast(data, schema)\n\n# > %{full_name: \"tada\"}\n```\n\n#","ref":"readme.html#field-name-alias","title":"Field name alias - Tarams","type":"extras"},{"doc":"You can specify a function similar to `cast_func` to manipulate data after casted.\nHowever data object passed to transform function is original data before casting.\n\n```elixir\ndata = %{status: 10}\n\nschema = %{\n    name: [type: :string, into: fn value -> {:ok, \"name: #{value}}\" end]\n}\n\nTarams.cast(data, schema)\n# > %{name: \"name: tada\"}\n\n```\n\n- Transform function can return tuple `{:ok, value}`, `{:error, message}` or value directly.\n\n```elixir\nschema = %{\n    value: [type: :integer, into: &to_string/1]\n}\n```","ref":"readme.html#convert-data","title":"Convert data - Tarams","type":"extras"},{"doc":"If you find a bug or want to improve something, please send a pull request. Thank you!","ref":"readme.html#contributors","title":"Contributors - Tarams","type":"extras"}]}